from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .assigned.assigned_request_builder import AssignedRequestBuilder
    from .closed.closed_request_builder import ClosedRequestBuilder
    from .grouped.grouped_request_builder import GroupedRequestBuilder

class ClashesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /bim360/clash/v3/containers/{containerId}/modelsets/{modelSetId}/clashes
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new ClashesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/bim360/clash/v3/containers/{containerId}/modelsets/{modelSetId}/clashes", path_parameters)
    
    @property
    def assigned(self) -> AssignedRequestBuilder:
        """
        The assigned property
        """
        from .assigned.assigned_request_builder import AssignedRequestBuilder

        return AssignedRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def closed(self) -> ClosedRequestBuilder:
        """
        The closed property
        """
        from .closed.closed_request_builder import ClosedRequestBuilder

        return ClosedRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def grouped(self) -> GroupedRequestBuilder:
        """
        The grouped property
        """
        from .grouped.grouped_request_builder import GroupedRequestBuilder

        return GroupedRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .content_views_get_response_results_view_sheet_view_calibration_units import ContentViewsGetResponse_results_view_SheetView_calibration_units

@dataclass
class ContentViewsGetResponse_results_view_SheetView_calibration(Parsable):
    """
    The sheet view calibration details.
    """
    # The scale used in the sheet view calibration.
    scale_factor: Optional[float] = None
    # The units used in the sheet view calibration.Possible values: ``FT_AND_DECIMAL_IN``, ``FT_AND_FRACTIONAL_IN``, ``M``, ``CM``, ``MM``.
    units: Optional[ContentViewsGetResponse_results_view_SheetView_calibration_units] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ContentViewsGetResponse_results_view_SheetView_calibration:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ContentViewsGetResponse_results_view_SheetView_calibration
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ContentViewsGetResponse_results_view_SheetView_calibration()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .content_views_get_response_results_view_sheet_view_calibration_units import ContentViewsGetResponse_results_view_SheetView_calibration_units

        from .content_views_get_response_results_view_sheet_view_calibration_units import ContentViewsGetResponse_results_view_SheetView_calibration_units

        fields: dict[str, Callable[[Any], None]] = {
            "scaleFactor": lambda n : setattr(self, 'scale_factor', n.get_float_value()),
            "units": lambda n : setattr(self, 'units', n.get_enum_value(ContentViewsGetResponse_results_view_SheetView_calibration_units)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_float_value("scaleFactor", self.scale_factor)
        writer.write_enum_value("units", self.units)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_response_patch_request_body_attachments_attachment_type import WithResponsePatchRequestBody_attachments_attachmentType

@dataclass
class WithResponsePatchRequestBody_attachments(Parsable):
    # The type of the attachment. Will always be: ``rfiResponse``
    attachment_type: Optional[WithResponsePatchRequestBody_attachments_attachmentType] = None
    # The user-facing name to show in the UI.
    display_name: Optional[str] = None
    # The internal file name.
    file_name: Optional[str] = None
    # The storage URN of the uploaded file.Use this value to generate a signed URL and download the file via the Data Management API.For more details, see the `Submit RFI Response </en/docs/acc/v1/tutorials/create-rfi-response/>`_ tutorial.
    storage_urn: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithResponsePatchRequestBody_attachments:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithResponsePatchRequestBody_attachments
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithResponsePatchRequestBody_attachments()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_response_patch_request_body_attachments_attachment_type import WithResponsePatchRequestBody_attachments_attachmentType

        from .with_response_patch_request_body_attachments_attachment_type import WithResponsePatchRequestBody_attachments_attachmentType

        fields: dict[str, Callable[[Any], None]] = {
            "attachmentType": lambda n : setattr(self, 'attachment_type', n.get_enum_value(WithResponsePatchRequestBody_attachments_attachmentType)),
            "displayName": lambda n : setattr(self, 'display_name', n.get_str_value()),
            "fileName": lambda n : setattr(self, 'file_name', n.get_str_value()),
            "storageUrn": lambda n : setattr(self, 'storage_urn', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_enum_value("attachmentType", self.attachment_type)
        writer.write_str_value("displayName", self.display_name)
        writer.write_str_value("fileName", self.file_name)
        writer.write_str_value("storageUrn", self.storage_urn)
    




from enum import Enum

class TakeoffTypesGetResponse_results_primaryQuantityDefinition_unitOfMeasure(str, Enum):
    EA = "EA",
    IN_ = "IN",
    LF = "LF",
    YD = "YD",
    SI = "SI",
    SF = "SF",
    SY = "SY",
    CI = "CI",
    CF = "CF",
    CY = "CY",
    LBS = "LBS",
    TON = "TON",
    MM = "MM",
    M = "M",
    M2 = "M2",
    M3 = "M3",
    KG = "KG",
    T = "T",




from enum import Enum

class WithRfiGetResponse_workflowType(str, Enum):
    US = "US",
    EU = "EU",
    Submitted = "submitted",
    OpenRev1 = "openRev1",
    AnsweredManager = "answeredManager",
    Status = "status",
    PreviousStatus = "previousStatus",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_custom_attribute_patch_request_body_data_type import WithCustomAttributePatchRequestBody_dataType
    from .with_custom_attribute_patch_request_body_default_value import WithCustomAttributePatchRequestBody_defaultValue

@dataclass
class WithCustomAttributePatchRequestBody(Parsable):
    # The data type that this custom attribute's value must take. Once set, the data type can't be changed.Possible values:- ``boolean``: ``true`` or ``false``- ``text``: a string- ``numeric``: a string that parses as a valid floating point number (not localized)- ``date``: an ISO8601 date string with no time, for example, ``2021-04-01``.- ``select``: a valid ID from the list of values defined by ``enumValues``.- ``multi_select``: an array of valid IDs from the list of values defined by ``enumValues``.
    data_type: Optional[WithCustomAttributePatchRequestBody_dataType] = None
    # The default value for this custom attribute if no value is specified on asset creation. If this field is notspecified, the custom attribute does not have a default value. The default value it takes can be any one ofthree possible value types, defined below.
    default_value: Optional[WithCustomAttributePatchRequestBody_defaultValue] = None
    # A description of the Asset custom attribute.Max length: 1000
    description: Optional[str] = None
    # The human-readable display name for the Asset custom attribute. This name appears when viewing the customattribute in the Assets UI, and must be unique within a project.Max length: 100
    display_name: Optional[str] = None
    # An array of string values that defines possible values that may be supplied when this custom attributes datatype (defined by ``dataType``) is set to ``select`` or ``multi_select``. This field is required when the datatype is set to ``select`` or ``multi_select``.
    enum_values: Optional[list[str]] = None
    # The maximum length that a text value can be for this custom attribute when creating or editing an asset.Setting a value for this field does not guarantee that existing or imported assets with this customattribute will have this attribute's value limited to this length. This field only applies when datatypeis set to ``text``. Default is ``250``, which is the maximum value this field can be set to.
    max_length_on_ingress: Optional[int] = None
    # Specifies whether or not this custom attribute is required when creating or editing an asset. If ``true``, thecustom attribute is required. If ``false``, it is not required. Setting this field to ``true`` does not guaranteethat existing or imported assets will have this custom attribute.
    required_on_ingress: Optional[bool] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithCustomAttributePatchRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithCustomAttributePatchRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithCustomAttributePatchRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_custom_attribute_patch_request_body_data_type import WithCustomAttributePatchRequestBody_dataType
        from .with_custom_attribute_patch_request_body_default_value import WithCustomAttributePatchRequestBody_defaultValue

        from .with_custom_attribute_patch_request_body_data_type import WithCustomAttributePatchRequestBody_dataType
        from .with_custom_attribute_patch_request_body_default_value import WithCustomAttributePatchRequestBody_defaultValue

        fields: dict[str, Callable[[Any], None]] = {
            "dataType": lambda n : setattr(self, 'data_type', n.get_enum_value(WithCustomAttributePatchRequestBody_dataType)),
            "defaultValue": lambda n : setattr(self, 'default_value', n.get_object_value(WithCustomAttributePatchRequestBody_defaultValue)),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "displayName": lambda n : setattr(self, 'display_name', n.get_str_value()),
            "enumValues": lambda n : setattr(self, 'enum_values', n.get_collection_of_primitive_values(str)),
            "maxLengthOnIngress": lambda n : setattr(self, 'max_length_on_ingress', n.get_int_value()),
            "requiredOnIngress": lambda n : setattr(self, 'required_on_ingress', n.get_bool_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_enum_value("dataType", self.data_type)
        writer.write_object_value("defaultValue", self.default_value)
        writer.write_str_value("description", self.description)
        writer.write_str_value("displayName", self.display_name)
        writer.write_collection_of_primitive_values("enumValues", self.enum_values)
        writer.write_int_value("maxLengthOnIngress", self.max_length_on_ingress)
        writer.write_bool_value("requiredOnIngress", self.required_on_ingress)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .comments_post_response_source import CommentsPostResponse_source

@dataclass
class CommentsPostResponse(Parsable):
    # The content of the comment.
    body: Optional[str] = None
    # The timestamp of the date and time the comment was created, in the following format: ``YYYY-MM-DDThh:mm:ss.sz``.
    created_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who created the comment.To check the name of the user, call `GET users </en/docs/acc/v1/reference/http/admin-v1-projects-projectId-users-GET>`_.
    created_by: Optional[str] = None
    # The unique identifier of the comment.
    id: Optional[str] = None
    # The ID of the RFI associated with this comment.
    rfi_id: Optional[str] = None
    # The source of the comment. Indicates how the comment was created.Possible values:- ``web`` – The comment was created through the web interface or API.- ``email`` – The comment was created by replying via email.
    source: Optional[CommentsPostResponse_source] = None
    # The timestamp of the date and time the comment was updated, in the following format: ``YYYY-MM-DDThh:mm:ss.sz``.
    updated_at: Optional[datetime.datetime] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> CommentsPostResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: CommentsPostResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return CommentsPostResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .comments_post_response_source import CommentsPostResponse_source

        from .comments_post_response_source import CommentsPostResponse_source

        fields: dict[str, Callable[[Any], None]] = {
            "body": lambda n : setattr(self, 'body', n.get_str_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "rfiId": lambda n : setattr(self, 'rfi_id', n.get_str_value()),
            "source": lambda n : setattr(self, 'source', n.get_enum_value(CommentsPostResponse_source)),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("body", self.body)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_str_value("id", self.id)
        writer.write_str_value("rfiId", self.rfi_id)
        writer.write_enum_value("source", self.source)
        writer.write_datetime_value("updatedAt", self.updated_at)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .naming_standards_get_response_definition_fields_options import NamingStandardsGetResponse_definition_fields_options
    from .naming_standards_get_response_definition_fields_type import NamingStandardsGetResponse_definition_fields_type

@dataclass
class NamingStandardsGetResponse_definition_fields(Parsable):
    # The ID of the field.
    attribute_id: Optional[float] = None
    # The default value of the field.For example, ``PROJ``.
    default_value: Optional[str] = None
    # A description of the field.Max length: 255
    description: Optional[str] = None
    # The maximum length of the field name.Max number of characters: 10
    max_length: Optional[float] = None
    # The minimum length of the field name.Min number of characters: 1
    min_length: Optional[float] = None
    # The name of the field.Max length: 32
    name: Optional[str] = None
    # ``true``: this field is required in the file naming convention.``false``: this field is not required in the file naming convention.
    optional: Optional[bool] = None
    # A drop-down list of possible values for the field.Only relevant for a field of type ``ARRAY`` (drop-down list).
    options: Optional[list[NamingStandardsGetResponse_definition_fields_options]] = None
    # The field type.Possible values:- ``ALPHANUMERIC``: Accepts all characters.- ``NONNUMERIC_TEXT``: Accepts all characters, excluding ``0-9``.- ``NUMERIC``: Accepts ``0-9``, comma ``','``, plus ``'+'``, minus ``'-'``, percent ``'%'``, period ``'.'``, underscore ``'_'``.- ``ARRAY``: A drop-down list.Note that the ``minLength`` and ``maxLength`` properties are not included in the response for the ``ARRAY`` type.
    type: Optional[NamingStandardsGetResponse_definition_fields_type] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> NamingStandardsGetResponse_definition_fields:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: NamingStandardsGetResponse_definition_fields
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return NamingStandardsGetResponse_definition_fields()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .naming_standards_get_response_definition_fields_options import NamingStandardsGetResponse_definition_fields_options
        from .naming_standards_get_response_definition_fields_type import NamingStandardsGetResponse_definition_fields_type

        from .naming_standards_get_response_definition_fields_options import NamingStandardsGetResponse_definition_fields_options
        from .naming_standards_get_response_definition_fields_type import NamingStandardsGetResponse_definition_fields_type

        fields: dict[str, Callable[[Any], None]] = {
            "attributeId": lambda n : setattr(self, 'attribute_id', n.get_float_value()),
            "defaultValue": lambda n : setattr(self, 'default_value', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "maxLength": lambda n : setattr(self, 'max_length', n.get_float_value()),
            "minLength": lambda n : setattr(self, 'min_length', n.get_float_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "optional": lambda n : setattr(self, 'optional', n.get_bool_value()),
            "options": lambda n : setattr(self, 'options', n.get_collection_of_object_values(NamingStandardsGetResponse_definition_fields_options)),
            "type": lambda n : setattr(self, 'type', n.get_enum_value(NamingStandardsGetResponse_definition_fields_type)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_float_value("attributeId", self.attribute_id)
        writer.write_str_value("defaultValue", self.default_value)
        writer.write_str_value("description", self.description)
        writer.write_float_value("maxLength", self.max_length)
        writer.write_float_value("minLength", self.min_length)
        writer.write_str_value("name", self.name)
        writer.write_bool_value("optional", self.optional)
        writer.write_collection_of_object_values("options", self.options)
        writer.write_enum_value("type", self.type)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .requirements_get_response_spec_sections import RequirementsGetResponse_specSections

@dataclass
class RequirementsGetResponse(Parsable):
    # The division code associated with the submittal. This is the equivalent to the *Division* column in the UI.
    division_code: Optional[str] = None
    # The division name associated with the submittal. This is equivalent to the name in the *Division* filter in the UI.
    division_name: Optional[str] = None
    # A list of specification divisions and groups.
    spec_sections: Optional[list[RequirementsGetResponse_specSections]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RequirementsGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RequirementsGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RequirementsGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .requirements_get_response_spec_sections import RequirementsGetResponse_specSections

        from .requirements_get_response_spec_sections import RequirementsGetResponse_specSections

        fields: dict[str, Callable[[Any], None]] = {
            "divisionCode": lambda n : setattr(self, 'division_code', n.get_str_value()),
            "divisionName": lambda n : setattr(self, 'division_name', n.get_str_value()),
            "specSections": lambda n : setattr(self, 'spec_sections', n.get_collection_of_object_values(RequirementsGetResponse_specSections)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("divisionCode", self.division_code)
        writer.write_str_value("divisionName", self.division_name)
        writer.write_collection_of_object_values("specSections", self.spec_sections)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .resources_get_response_results_custom_attributes_type import ResourcesGetResponse_results_customAttributes_type

@dataclass
class ResourcesGetResponse_results_customAttributes(Parsable):
    # The unique identifier of the custom attribute.
    id: Optional[int] = None
    # The name of the custom attribute.
    name: Optional[str] = None
    # The data type of the custom attribute. Possible values:- ``string`` – Text field.- ``date`` – Date field.- ``array`` – Drop-down list.
    type: Optional[ResourcesGetResponse_results_customAttributes_type] = None
    # The value of the custom attribute.
    value: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ResourcesGetResponse_results_customAttributes:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ResourcesGetResponse_results_customAttributes
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ResourcesGetResponse_results_customAttributes()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .resources_get_response_results_custom_attributes_type import ResourcesGetResponse_results_customAttributes_type

        from .resources_get_response_results_custom_attributes_type import ResourcesGetResponse_results_customAttributes_type

        fields: dict[str, Callable[[Any], None]] = {
            "id": lambda n : setattr(self, 'id', n.get_int_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "type": lambda n : setattr(self, 'type', n.get_enum_value(ResourcesGetResponse_results_customAttributes_type)),
            "value": lambda n : setattr(self, 'value', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_int_value("id", self.id)
        writer.write_str_value("name", self.name)
        writer.write_enum_value("type", self.type)
        writer.write_str_value("value", self.value)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID
from warnings import warn

if TYPE_CHECKING:
    from .item.with_review_item_request_builder import WithReviewItemRequestBuilder
    from .reviews_get_response import ReviewsGetResponse
    from .reviews_post_request_body import ReviewsPostRequestBody
    from .reviews_post_response import ReviewsPostResponse

class ReviewsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/reviews/v1/projects/{projectId}/reviews
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new ReviewsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/reviews/v1/projects/{projectId}/reviews{?filter%5BarchivedAt%5D*,filter%5BarchivedBy%5D*,filter%5Barchived%5D*,filter%5BcreatedAt%5D*,filter%5BcurrentStepDueDate%5D*,filter%5BfinishedAt%5D*,filter%5Bname%5D*,filter%5BnextActionByCompany%5D*,filter%5BnextActionByRole%5D*,filter%5BnextActionByUser%5D*,filter%5BsequenceId%5D*,filter%5Bstatus%5D*,filter%5BupdatedAt%5D*,filter%5BworkflowId%5D*,limit*,offset*,sort*}", path_parameters)
    
    def by_review_id(self,review_id: UUID) -> WithReviewItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.reviews.v1.projects.item.reviews.item collection
        param review_id: The unique ID of the review.It must be in UUID format — not the numeric sequence ID shown in the Reviews UI. To find the review ID, call `GET reviews <en/docs/acc/v1/reference/http/reviews-reviews-GET/>`_.
        Returns: WithReviewItemRequestBuilder
        """
        if review_id is None:
            raise TypeError("review_id cannot be null.")
        from .item.with_review_item_request_builder import WithReviewItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["reviewId"] = review_id
        return WithReviewItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[ReviewsRequestBuilderGetQueryParameters]] = None) -> Optional[ReviewsGetResponse]:
        """
        Retrieves the list of reviews created in the specified project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[ReviewsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .reviews_get_response import ReviewsGetResponse

        return await self.request_adapter.send_async(request_info, ReviewsGetResponse, None)
    
    async def post(self,body: ReviewsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[ReviewsPostResponse]:
        """
        Creates a new review in the specified project using an existing approval workflow.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[ReviewsPostResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .reviews_post_response import ReviewsPostResponse

        return await self.request_adapter.send_async(request_info, ReviewsPostResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[ReviewsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves the list of reviews created in the specified project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: ReviewsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Creates a new review in the specified project using an existing approval workflow.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> ReviewsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: ReviewsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return ReviewsRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class ReviewsRequestBuilderGetQueryParameters():
        """
        Retrieves the list of reviews created in the specified project.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filterarchived":
                return "filter%5Barchived%5D"
            if original_name == "filterarchived_at":
                return "filter%5BarchivedAt%5D"
            if original_name == "filterarchived_by":
                return "filter%5BarchivedBy%5D"
            if original_name == "filtercreated_at":
                return "filter%5BcreatedAt%5D"
            if original_name == "filtercurrent_step_due_date":
                return "filter%5BcurrentStepDueDate%5D"
            if original_name == "filterfinished_at":
                return "filter%5BfinishedAt%5D"
            if original_name == "filtername":
                return "filter%5Bname%5D"
            if original_name == "filternext_action_by_company":
                return "filter%5BnextActionByCompany%5D"
            if original_name == "filternext_action_by_role":
                return "filter%5BnextActionByRole%5D"
            if original_name == "filternext_action_by_user":
                return "filter%5BnextActionByUser%5D"
            if original_name == "filtersequence_id":
                return "filter%5BsequenceId%5D"
            if original_name == "filterstatus":
                return "filter%5Bstatus%5D"
            if original_name == "filterupdated_at":
                return "filter%5BupdatedAt%5D"
            if original_name == "filterworkflow_id":
                return "filter%5BworkflowId%5D"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # Filter by archive status in URL-encoded format.``true``: retrieves only archived reviews.``false``: retrieves only active (non-archived) reviews.If omitted, only active reviews are retrieved.For example: ``filter[archived]=false``.
        filterarchived: Optional[bool] = None

        # Filter by the date the review was archived, in URL-encoded format.It only applies if ``filter[archived]=true``.Provide a date range using the format ``startDate..endDate``.Both values must be in ISO 8601 format.For example: ``filter[archivedAt]=2023-06-01..2023-06-30``.
        filterarchived_at: Optional[str] = None

        # Filter by the Autodesk ID of the user who archived the review, in URL-encoded format. To find the ID, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET/>`_.It only takes effect when ``filter[archived]=true`` is also set.For example: ``filter[archivedBy]=A96JX8NUKRLVFWSR``.
        filterarchived_by: Optional[str] = None

        # Filter by review creation date in URL-encoded format.Provide a date range using the format ``startDate..endDate``.Both values must be in ISO 8601 format.For example: ``filter[createdAt]=2023-06-01..2023-06-30``
        filtercreated_at: Optional[str] = None

        # Filter by the due date of the current review step in URL-encoded format.Provide a date range using the format ``startDate..endDate``.Both values must be in ISO 8601 format.For example: ``filter[updatedAt]=2023-06-01..2023-06-30``.
        filtercurrent_step_due_date: Optional[str] = None

        # Filter by the date the review was finished, in URL-encoded format.Provide a date range using the format ``startDate..endDate``.Both values must be in ISO 8601 format.For example: ``filter[finishedAt]=2023-06-01..2023-06-30``.
        filterfinished_at: Optional[str] = None

        # Filter by review name in URL-encoded format.Retrieves reviews with names that contain the specified string (not case-sensitive).For example: ``filter[name]=Apartment`` retrieves reviews like ``Apartment Block A`` and ``apartment_rendering``.
        filtername: Optional[str] = None

        # Filter by Autodesk ID of a company responsible for the next action, in URL-encoded format.For example: ``filter[nextActionByCompany]=81768771``.
        filternext_action_by_company: Optional[str] = None

        # Filter by Autodesk ID of a role responsible for the next action, in URL-encoded format.For example: ``filter[nextActionByRole]=1572818``.
        filternext_action_by_role: Optional[str] = None

        # Filter by Autodesk ID of a user responsible for the next action, in URL-encoded format.This includes reviews assigned directly to the user or to their role or company.For example: ``filter[nextActionByUser]=A96JX8NUKRLVFWSR``.
        filternext_action_by_user: Optional[str] = None

        # Filter by review sequence ID in URL-encoded format.Retrieves reviews with sequence IDs that partially match the specified number.For example: ``filter[sequenceId]=11`` may retrieve ``113`` and ``211``.
        filtersequence_id: Optional[int] = None

        # Filter by the review status in URL-encoded format.Possible values: ``OPEN``, ``CLOSED``, ``VOID``, ``FAILED``.For example: ``filter[status]=OPEN``.Reviews with status ``FAILED`` are only visible to project administrators.
        filterstatus: Optional[str] = None

        # Filter by the review’s last updated date in URL-encoded format.Provide a date range using the format ``startDate..endDate``.Both values must be in ISO 8601 format.For example: ``filter[updatedAt]=2023-06-01..2023-06-30``.
        filterupdated_at: Optional[str] = None

        # Filter by a specific approval workflow ID in URL-encoded format.For example: ``filter[workflowId]=497f6eca-6276-4993-bfeb-53cbbbba6f08``.
        filterworkflow_id: Optional[UUID] = None

        # The maximum number of reviews to retrieve.Possible values: ``1-50``. Maximum: ``50``. Default: ``50``. For example: ``limit=2``.
        limit: Optional[int] = None

        # The index of the first result to return (zero-based).Default: ``0``. For example: ``offset=10``.
        offset: Optional[int] = None

        # Sorts the results by a single field.Use the format ``fieldName`` (ascending) or ``fieldName desc`` (descending).If no direction is specified, sorting defaults to ascending.Possible values: ``name``, ``status``, ``sequenceId``, ``currentStepDueDate``, ``createdAt``, ``finishedAt``.For example: ``sort=createdAt desc``.
        sort: Optional[str] = None

    
    @dataclass
    class ReviewsRequestBuilderGetRequestConfiguration(RequestConfiguration[ReviewsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class ReviewsRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_rfi_patch_request_body_assigned_to import WithRfiPatchRequestBody_assignedTo
    from .with_rfi_patch_request_body_attachments import WithRfiPatchRequestBody_attachments
    from .with_rfi_patch_request_body_co_reviewers import WithRfiPatchRequestBody_coReviewers
    from .with_rfi_patch_request_body_custom_attributes import WithRfiPatchRequestBody_customAttributes
    from .with_rfi_patch_request_body_official_response_status import WithRfiPatchRequestBody_officialResponseStatus
    from .with_rfi_patch_request_body_return_for_reviewer_list import WithRfiPatchRequestBody_returnForReviewerList
    from .with_rfi_patch_request_body_status import WithRfiPatchRequestBody_status
    from .with_rfi_patch_request_body_watchers import WithRfiPatchRequestBody_watchers

@dataclass
class WithRfiPatchRequestBody(Parsable):
    # A list of response IDs to include in the final official response.Use this to reference earlier draft or submitted responses that should appear in the final answer.Only used when closing the RFI.
    aggregated_response_ids: Optional[list[str]] = None
    # The list of users assigned to the RFI.
    assigned_to: Optional[list[WithRfiPatchRequestBody_assignedTo]] = None
    # A list of attachments to include in the official RFI response when closing the RFI.
    attachments: Optional[list[WithRfiPatchRequestBody_attachments]] = None
    # Whether to generate an RFI report for the target projects.
    bridge_generate_rfi_report: Optional[bool] = None
    # A list of predefined categories to assign to the RFI.Categories help group RFIs for filtering and reporting. Each value must match a category configured in the project’s RFI settings. Categories are case-sensitive and project-specific.RFI categories are configured in Project Admin and may differ between projects. Call `GET rfi-types </en/docs/acc/v1/reference/http/rfis-rfi-types-GET>`_ to retrieve the allowed values for this field.Some possible values: ``Code Compliance``, ``Constructability``, ``Design Coordination``, ``Documentation Conflict``, ``Documentation Incomplete``, ``Field condition``, ``Other``.
    category: Optional[list[str]] = None
    # A list of reviewers assigned to the RFI. Each entry may represent a user, role, or company.
    co_reviewers: Optional[list[WithRfiPatchRequestBody_coReviewers]] = None
    # The default cost impact value for new RFIs of this type.Possible values: ``null``, ``Yes``, ``No``, ``Unknown``.To check whether cost impact options are enabled and to retrieve the default value, call `GET rfi-types <en/docs/acc/v1/reference/http/rfis-RFI-types-GET/>`_.
    cost_impact: Optional[str] = None
    # A list of custom attributes associated with the RFI.
    custom_attributes: Optional[list[WithRfiPatchRequestBody_customAttributes]] = None
    # The discipline associated with the RFI. To retrieve the supported values for the current project, call `GET rfi-types </en/docs/acc/v1/reference/http/rfis-rfi-types-GET>`_.Some possible values: ``Building Management System``, ``Electrical Substation``, ``Security``, ``Audio Visual``, ``Food Service``, ``Fire Alarm``, ``Power Systems``, ``Design Systems Integrator``, ``Signage``, ``Pathways``, ``Cabling``, ``Networks``, ``Distributed Antenna System``, ``Lighting``, ``Vertical Transportation``, ``Roofing``, ``Architectural``, ``Civil/Site``, ``Concrete``, ``Electrical``, ``Exterior Envelope``, ``Fire Protection``, ``Interior/Finishes``, ``Landscaping``, ``Masonry``, ``Mechanical``, ``Plumbing``, ``Structural``, ``Other``.'
    discipline: Optional[list[str]] = None
    # The response due date, in the following format: ``YYYY-MM-DDThh:mm:ss.sz``.The due date is not auto-calculated by the API. To determine the recommended due date for a new RFI, call `GET rfi-types </en/docs/acc/v1/reference/http/rfis-rfi-types-GET>`_ and use the value of the ``dueDateOffset`` field to add days to the current date.
    due_date: Optional[str] = None
    # The default text for the Location field when creating a new RFI.Note that the API does not auto-populate this value.Clients are responsible for applying the default if desired.To retrieve the default value configured for this field, call `GET rfi-types </en/docs/acc/v1/reference/http/rfis-rfi-types-GET>`_.
    location_description: Optional[str] = None
    # A list of location IDs to associate with the RFI. These must be valid UUIDs from the project’s Location Breakdown Structure (LBS).To retrieve them, call `GET nodes </en/docs/acc/v1/reference/http/projects-projectId-locations-nodes-GET>`_.
    locations: Optional[list[str]] = None
    # The text of the final official response to the RFI.Use this field when closing the RFI to submit the final answer.This field is not used in other status transitions.
    official_response: Optional[str] = None
    # The status of the official response. Possible values: ``answered``, ``rejected``.Set this field when closing the RFI to mark the final response as ``answered`` or ``rejected``.This field is not used in other status transitions.
    official_response_status: Optional[WithRfiPatchRequestBody_officialResponseStatus] = None
    # The default priority for new RFIs of this type.The available priority values are configured in Project Admin.If no default is set, this field is ``null``.Note that the API does not auto-populate this value when creating an RFI.Clients are responsible for applying the default if desired.The valid priority options can be retrieved by calling `GET rfi-types <en/docs/acc/v1/reference/http/rfis-RFI-types-GET/>_. Some possible values: ``null``, ``High``, ``Normal``, ``Low``.
    priority: Optional[str] = None
    # The RFI question.
    question: Optional[str] = None
    # A free-text field for cross-referencing the RFI with other systems or documents.Max length: 20
    reference: Optional[str] = None
    # A list of watchers who are notified about changes to the RFI. Each entry may represent a user, role, or company.
    return_for_reviewer_list: Optional[list[WithRfiPatchRequestBody_returnForReviewerList]] = None
    # The default schedule impact value for new RFIs of this type.Possible values: ``null``, ``Yes``, ``No``, ``Unknown``.To verify whether schedule impact tracking is enabled for the project and what the default value is, call `GET rfi-types <en/docs/acc/v1/reference/http/rfis-RFI-types-GET/>`_.
    schedule_impact: Optional[str] = None
    # The current status of the RFI. Available values depend on the RFI’s workflow type:- For single-reviewer workflows (US):Possible values:``draft``, ``submitted``, ``open``, ``answered``, ``rejected``, ``closed``, ``void``.- For multi-reviewer workflows (EMEA):Possible values:``draft``, ``submitted``, ``openRev1`` (manager), ``openRev2`` (reviewers), ``answeredRev1``, ``answeredManager``, ``closed``, ``void``.To determine the workflow type, call `GET users/me </en/docs/acc/v1/reference/http/rfis-users-me-GET/>`_ and check the ``workflowType`` value.For details on RFI workflows in the ACC UI, see `About RFI  Workflows – Autodesk Help <https://help.autodesk.com/view/BUILD/ENU/?guid=RFI_Workflow_Setup>`_.
    status: Optional[WithRfiPatchRequestBody_status] = None
    # The suggested answer for the RFI.
    suggested_answer: Optional[str] = None
    # The title of the RFI.
    title: Optional[str] = None
    # A list of watchers who are notified about changes to the RFI. Each entry may represent a user, role, or company.
    watchers: Optional[list[WithRfiPatchRequestBody_watchers]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithRfiPatchRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithRfiPatchRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithRfiPatchRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_rfi_patch_request_body_assigned_to import WithRfiPatchRequestBody_assignedTo
        from .with_rfi_patch_request_body_attachments import WithRfiPatchRequestBody_attachments
        from .with_rfi_patch_request_body_co_reviewers import WithRfiPatchRequestBody_coReviewers
        from .with_rfi_patch_request_body_custom_attributes import WithRfiPatchRequestBody_customAttributes
        from .with_rfi_patch_request_body_official_response_status import WithRfiPatchRequestBody_officialResponseStatus
        from .with_rfi_patch_request_body_return_for_reviewer_list import WithRfiPatchRequestBody_returnForReviewerList
        from .with_rfi_patch_request_body_status import WithRfiPatchRequestBody_status
        from .with_rfi_patch_request_body_watchers import WithRfiPatchRequestBody_watchers

        from .with_rfi_patch_request_body_assigned_to import WithRfiPatchRequestBody_assignedTo
        from .with_rfi_patch_request_body_attachments import WithRfiPatchRequestBody_attachments
        from .with_rfi_patch_request_body_co_reviewers import WithRfiPatchRequestBody_coReviewers
        from .with_rfi_patch_request_body_custom_attributes import WithRfiPatchRequestBody_customAttributes
        from .with_rfi_patch_request_body_official_response_status import WithRfiPatchRequestBody_officialResponseStatus
        from .with_rfi_patch_request_body_return_for_reviewer_list import WithRfiPatchRequestBody_returnForReviewerList
        from .with_rfi_patch_request_body_status import WithRfiPatchRequestBody_status
        from .with_rfi_patch_request_body_watchers import WithRfiPatchRequestBody_watchers

        fields: dict[str, Callable[[Any], None]] = {
            "aggregatedResponseIds": lambda n : setattr(self, 'aggregated_response_ids', n.get_collection_of_primitive_values(str)),
            "assignedTo": lambda n : setattr(self, 'assigned_to', n.get_collection_of_object_values(WithRfiPatchRequestBody_assignedTo)),
            "attachments": lambda n : setattr(self, 'attachments', n.get_collection_of_object_values(WithRfiPatchRequestBody_attachments)),
            "bridgeGenerateRfiReport": lambda n : setattr(self, 'bridge_generate_rfi_report', n.get_bool_value()),
            "category": lambda n : setattr(self, 'category', n.get_collection_of_primitive_values(str)),
            "coReviewers": lambda n : setattr(self, 'co_reviewers', n.get_collection_of_object_values(WithRfiPatchRequestBody_coReviewers)),
            "costImpact": lambda n : setattr(self, 'cost_impact', n.get_str_value()),
            "customAttributes": lambda n : setattr(self, 'custom_attributes', n.get_collection_of_object_values(WithRfiPatchRequestBody_customAttributes)),
            "discipline": lambda n : setattr(self, 'discipline', n.get_collection_of_primitive_values(str)),
            "dueDate": lambda n : setattr(self, 'due_date', n.get_str_value()),
            "locationDescription": lambda n : setattr(self, 'location_description', n.get_str_value()),
            "locations": lambda n : setattr(self, 'locations', n.get_collection_of_primitive_values(str)),
            "officialResponse": lambda n : setattr(self, 'official_response', n.get_str_value()),
            "officialResponseStatus": lambda n : setattr(self, 'official_response_status', n.get_enum_value(WithRfiPatchRequestBody_officialResponseStatus)),
            "priority": lambda n : setattr(self, 'priority', n.get_str_value()),
            "question": lambda n : setattr(self, 'question', n.get_str_value()),
            "reference": lambda n : setattr(self, 'reference', n.get_str_value()),
            "returnForReviewerList": lambda n : setattr(self, 'return_for_reviewer_list', n.get_collection_of_object_values(WithRfiPatchRequestBody_returnForReviewerList)),
            "scheduleImpact": lambda n : setattr(self, 'schedule_impact', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(WithRfiPatchRequestBody_status)),
            "suggestedAnswer": lambda n : setattr(self, 'suggested_answer', n.get_str_value()),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
            "watchers": lambda n : setattr(self, 'watchers', n.get_collection_of_object_values(WithRfiPatchRequestBody_watchers)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_primitive_values("aggregatedResponseIds", self.aggregated_response_ids)
        writer.write_collection_of_object_values("assignedTo", self.assigned_to)
        writer.write_collection_of_object_values("attachments", self.attachments)
        writer.write_bool_value("bridgeGenerateRfiReport", self.bridge_generate_rfi_report)
        writer.write_collection_of_primitive_values("category", self.category)
        writer.write_collection_of_object_values("coReviewers", self.co_reviewers)
        writer.write_str_value("costImpact", self.cost_impact)
        writer.write_collection_of_object_values("customAttributes", self.custom_attributes)
        writer.write_collection_of_primitive_values("discipline", self.discipline)
        writer.write_str_value("dueDate", self.due_date)
        writer.write_str_value("locationDescription", self.location_description)
        writer.write_collection_of_primitive_values("locations", self.locations)
        writer.write_str_value("officialResponse", self.official_response)
        writer.write_enum_value("officialResponseStatus", self.official_response_status)
        writer.write_str_value("priority", self.priority)
        writer.write_str_value("question", self.question)
        writer.write_str_value("reference", self.reference)
        writer.write_collection_of_object_values("returnForReviewerList", self.return_for_reviewer_list)
        writer.write_str_value("scheduleImpact", self.schedule_impact)
        writer.write_enum_value("status", self.status)
        writer.write_str_value("suggestedAnswer", self.suggested_answer)
        writer.write_str_value("title", self.title)
        writer.write_collection_of_object_values("watchers", self.watchers)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_project_get_response_products_key import WithProjectGetResponse_products_key
    from .with_project_get_response_products_language import WithProjectGetResponse_products_language
    from .with_project_get_response_products_status import WithProjectGetResponse_products_status

@dataclass
class WithProjectGetResponse_products(Parsable):
    # The URL of the icon associated with the product.
    icon: Optional[str] = None
    # A machine-readable identifier for the product (e.g., docs, build).Each product has a unique key used throughout the API for identification, filtering, and integration logic (e.g., in query parameters like ``filter[key]``).Possible values:ACC - ``autoSpecs``, ``build``, ``cost``, ``designCollaboration``, ``docs``, ``insight``, ``modelCoordination``, ``projectAdministration``, and ``takeoff``.BIM 360 - ``assets``, ``costManagement``, ``designCollaboration``, ``documentManagement``, ``field``, ``fieldManagement``, ``glue``, ``insight``, ``modelCoordination``, ``plan``, ``projectAdministration``, ``projectHome``, ``projectManagement``, and ``quantification``.Note that this endpoint returns only ACC products. Other endpoints, such as `GET projects </en/docs/acc/v1/reference/http/admin-accountsaccountidprojects-GET/>`_ and `GET projects/:projectId </en/docs/acc/v1/reference/http/admin-projects-projectId-GET/>`_, may return both ACC and BIM 360 projects. In those responses, product keys may include BIM 360 values.
    key: Optional[WithProjectGetResponse_products_key] = None
    # The language for the project. Only valid for the ``field`` product.Possible values: ``en``, ``de``, ``nl``, ``zh``, ``de-CH``
    language: Optional[WithProjectGetResponse_products_language] = None
    # The name of the product.
    name: Optional[str] = None
    # The current status of the product. Possible values:- ``activating``: Product activation is in progress.- ``activationFailed``: Product activation has failed.- ``active``: Product activation is completed.- ``deactivating``: Product deactivation is in progress. (Applicable to BIM 360 only)- ``deactivationFailed``: Product deactivation has failed. (Applicable to BIM 360 only)- ``inactive``: Product deactivation is completed. (Applicable to BIM 360 only)- ``available``: Product is available for activation. (Applicable to BIM 360 only)
    status: Optional[WithProjectGetResponse_products_status] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithProjectGetResponse_products:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithProjectGetResponse_products
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithProjectGetResponse_products()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_project_get_response_products_key import WithProjectGetResponse_products_key
        from .with_project_get_response_products_language import WithProjectGetResponse_products_language
        from .with_project_get_response_products_status import WithProjectGetResponse_products_status

        from .with_project_get_response_products_key import WithProjectGetResponse_products_key
        from .with_project_get_response_products_language import WithProjectGetResponse_products_language
        from .with_project_get_response_products_status import WithProjectGetResponse_products_status

        fields: dict[str, Callable[[Any], None]] = {
            "icon": lambda n : setattr(self, 'icon', n.get_str_value()),
            "key": lambda n : setattr(self, 'key', n.get_enum_value(WithProjectGetResponse_products_key)),
            "language": lambda n : setattr(self, 'language', n.get_enum_value(WithProjectGetResponse_products_language)),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(WithProjectGetResponse_products_status)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("icon", self.icon)
        writer.write_enum_value("key", self.key)
        writer.write_enum_value("language", self.language)
        writer.write_str_value("name", self.name)
        writer.write_enum_value("status", self.status)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .clashes.clashes_request_builder import ClashesRequestBuilder
    from .clashes_reopen.clashes_reopen_request_builder import ClashesReopenRequestBuilder
    from .screenshots.screenshots_request_builder import ScreenshotsRequestBuilder
    from .tests.tests_request_builder import TestsRequestBuilder
    from .versions.versions_request_builder import VersionsRequestBuilder

class WithModelSetItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /bim360/clash/v3/containers/{containerId}/modelsets/{modelSetId}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new WithModelSetItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/bim360/clash/v3/containers/{containerId}/modelsets/{modelSetId}", path_parameters)
    
    @property
    def clashes(self) -> ClashesRequestBuilder:
        """
        The clashes property
        """
        from .clashes.clashes_request_builder import ClashesRequestBuilder

        return ClashesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def clashes_reopen(self) -> ClashesReopenRequestBuilder:
        """
        The clashesReopen property
        """
        from .clashes_reopen.clashes_reopen_request_builder import ClashesReopenRequestBuilder

        return ClashesReopenRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def screenshots(self) -> ScreenshotsRequestBuilder:
        """
        The screenshots property
        """
        from .screenshots.screenshots_request_builder import ScreenshotsRequestBuilder

        return ScreenshotsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def tests(self) -> TestsRequestBuilder:
        """
        The tests property
        """
        from .tests.tests_request_builder import TestsRequestBuilder

        return TestsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def versions(self) -> VersionsRequestBuilder:
        """
        The versions property
        """
        from .versions.versions_request_builder import VersionsRequestBuilder

        return VersionsRequestBuilder(self.request_adapter, self.path_parameters)
    




from enum import Enum

class SearchRfisPostResponse_results_previousStatus(str, Enum):
    Draft = "draft",
    Submitted = "submitted",
    Open = "open",
    Answered = "answered",
    Rejected = "rejected",
    Closed = "closed",
    Void = "void",
    OpenRev1 = "openRev1",
    OpenRev2 = "openRev2",
    AnsweredRev1 = "answeredRev1",
    AnsweredManager = "answeredManager",
    WorkflowType = "workflowType",




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID
from warnings import warn

if TYPE_CHECKING:
    from .issues_get_response import IssuesGetResponse
    from .issues_post_request_body import IssuesPostRequestBody
    from .issues_post_response import IssuesPostResponse
    from .item.with_issue_item_request_builder import WithIssueItemRequestBuilder

class IssuesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/issues/v1/projects/{projectId}/issues
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new IssuesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/issues/v1/projects/{projectId}/issues{?fields*,filter%5BassignedToType%5D*,filter%5BassignedTo%5D*,filter%5BclosedAt%5D*,filter%5BclosedBy%5D*,filter%5BcreatedAt%5D*,filter%5BcreatedBy%5D*,filter%5BcustomAttributes%5D*,filter%5Bdeleted%5D*,filter%5BdisplayId%5D*,filter%5BdueDate%5D*,filter%5Bid%5D*,filter%5BissueSubtypeId%5D*,filter%5BissueTypeId%5D*,filter%5BlinkedDocumentUrn%5D*,filter%5BlocationId%5D*,filter%5BrootCauseId%5D*,filter%5Bsearch%5D*,filter%5BstartDate%5D*,filter%5Bstatus%5D*,filter%5BsubLocationId%5D*,filter%5BupdatedAt%5D*,filter%5BupdatedBy%5D*,filter%5Bvalid%5D*,limit*,offset*,sortBy*}", path_parameters)
    
    def by_issue_id(self,issue_id: UUID) -> WithIssueItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.issues.v1.projects.item.issues.item collection
        param issue_id: The unique identifier of the issue. To find the ID, call `GET issues </en/docs/acc/v1/reference/http/issues-issues-GET/>`_.
        Returns: WithIssueItemRequestBuilder
        """
        if issue_id is None:
            raise TypeError("issue_id cannot be null.")
        from .item.with_issue_item_request_builder import WithIssueItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["issueId"] = issue_id
        return WithIssueItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[IssuesRequestBuilderGetQueryParameters]] = None) -> Optional[IssuesGetResponse]:
        """
        Retrieves information about all the issues in a project, including details about their associated comments and attachments.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[IssuesGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .issues_get_response import IssuesGetResponse

        return await self.request_adapter.send_async(request_info, IssuesGetResponse, None)
    
    async def post(self,body: IssuesPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[IssuesPostResponse]:
        """
        Adds an issue to a project.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[IssuesPostResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .issues_post_response import IssuesPostResponse

        return await self.request_adapter.send_async(request_info, IssuesPostResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[IssuesRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves information about all the issues in a project, including details about their associated comments and attachments.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: IssuesPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Adds an issue to a project.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> IssuesRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: IssuesRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return IssuesRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class IssuesRequestBuilderGetQueryParameters():
        """
        Retrieves information about all the issues in a project, including details about their associated comments and attachments.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filterassigned_to":
                return "filter%5BassignedTo%5D"
            if original_name == "filterassigned_to_type":
                return "filter%5BassignedToType%5D"
            if original_name == "filterclosed_at":
                return "filter%5BclosedAt%5D"
            if original_name == "filterclosed_by":
                return "filter%5BclosedBy%5D"
            if original_name == "filtercreated_at":
                return "filter%5BcreatedAt%5D"
            if original_name == "filtercreated_by":
                return "filter%5BcreatedBy%5D"
            if original_name == "filtercustom_attributes":
                return "filter%5BcustomAttributes%5D"
            if original_name == "filterdeleted":
                return "filter%5Bdeleted%5D"
            if original_name == "filterdisplay_id":
                return "filter%5BdisplayId%5D"
            if original_name == "filterdue_date":
                return "filter%5BdueDate%5D"
            if original_name == "filterid":
                return "filter%5Bid%5D"
            if original_name == "filterissue_subtype_id":
                return "filter%5BissueSubtypeId%5D"
            if original_name == "filterissue_type_id":
                return "filter%5BissueTypeId%5D"
            if original_name == "filterlinked_document_urn":
                return "filter%5BlinkedDocumentUrn%5D"
            if original_name == "filterlocation_id":
                return "filter%5BlocationId%5D"
            if original_name == "filterroot_cause_id":
                return "filter%5BrootCauseId%5D"
            if original_name == "filtersearch":
                return "filter%5Bsearch%5D"
            if original_name == "filterstart_date":
                return "filter%5BstartDate%5D"
            if original_name == "filterstatus":
                return "filter%5Bstatus%5D"
            if original_name == "filtersub_location_id":
                return "filter%5BsubLocationId%5D"
            if original_name == "filterupdated_at":
                return "filter%5BupdatedAt%5D"
            if original_name == "filterupdated_by":
                return "filter%5BupdatedBy%5D"
            if original_name == "filtervalid":
                return "filter%5Bvalid%5D"
            if original_name == "sort_by":
                return "sortBy"
            if original_name == "fields":
                return "fields"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            return original_name
        
        # Return only specific fields in issue object. Separate multiple values with commas. Fields which will be returned in any case: ``id``, ``title``, ``status``, ``issueTypeId``. Possible values: ``id``, ``displayId``, ``title``, ``description``, ``issueTypeId``, ``issueSubtypeId``, ``status``, ``assignedTo``, ``assignedToType``, ``dueDate``, ``startDate``, ``locationId``, ``locationDetails``, ``rootCauseTitle``, ``rootCauseId``, ``permittedStatuses``, ``permittedAttributes``, ``permittedActions``, ``published``, ``commentCount``, ``openedBy``, ``openedAt``, ``closedBy``, ``closedAt``, ``createdBy``, ``createdAt``, ``updatedBy``, ``updatedAt``, ``customAttributes``.
        fields: Optional[list[str]] = None

        # Filter issues by the unique Autodesk ID of the User/Company/Role identifier of the current assignee of this issue. Separate multiple values with commas. For Example: ``A3RGM375QTZ7``.
        filterassigned_to: Optional[list[str]] = None

        # Filter issues by the type of the current assignee of this issue. Separate multiple values with commas. Possible values: Possible values: ``user``, ``company``, ``role``, ``null``. For Example: ``user``.
        filterassigned_to_type: Optional[str] = None

        # Filter issues closed at the specified date and time, in one of the following URL-encoded formats: YYYY-MM-DDThh:mm:ss.sz or YYYY-MM-DD. Separate multiple values with commas. We support the following filtering options:- Date range: e.g., ``2022-03-02..2022-03-03`` or ``2022-02-28T22:00:00.000Z..2022-03-28T22:00:00.000Z``- Specific day: e.g., ``2022-03-02`` or ``2022-02-28T22:00:00.000Z``- Specific start date: e.g.,  ``2022-03-02..`` or ``2022-02-28T22:00:00.000Z..``- Specific end date: e.g., ``..2022-03-02`` or ``..2022-02-28T22:00:00.000Z``For more details, see `JSON API Filtering <http://jsonapi.org/format/#fetching-filtering>`_.
        filterclosed_at: Optional[datetime.datetime] = None

        # Filter issues by the unique identifier of the user who closed the issue. Separate multiple values with commas. For Example: ``A3RGM375QTZ7``.
        filterclosed_by: Optional[list[str]] = None

        # Filter issues created at the specified date and time, in one of the following URL-encoded formats: YYYY-MM-DDThh:mm:ss.sz or YYYY-MM-DD. Separate multiple values with commas. We support the following filtering options:- Date range: e.g., ``2022-03-02..2022-03-03`` or ``2022-02-28T22:00:00.000Z..2022-03-28T22:00:00.000Z``- Specific day: e.g., ``2022-03-02`` or ``2022-02-28T22:00:00.000Z``- Specific start date: e.g.,  ``2022-03-02..`` or ``2022-02-28T22:00:00.000Z..``- Specific end date: e.g., ``..2022-03-02`` or ``..2022-02-28T22:00:00.000Z``For more details, see `JSON API Filtering <http://jsonapi.org/format/#fetching-filtering>`_.
        filtercreated_at: Optional[datetime.datetime] = None

        # Filter issues by the unique identifier of the user who created the issue. Separate multiple values with commas. For Example: ``A3RGM375QTZ7``.
        filtercreated_by: Optional[list[str]] = None

        # Filter issues by the custom attributes. Each custom attribute filter should be defined by it's uuid. For example: ``filter[customAttributes][f227d940-ae9b-4722-9297-389f4411f010]=1,2,3``. Separate multiple values with commas.
        filtercustom_attributes: Optional[list[str]] = None

        # Filter deleted issues. For example, ``filter[deleted]=true``. If set to ``true`` it only returns deleted issues. If set to ``false`` it only returns undeleted issues. Note that we do not currently support returning both deleted and undeleted issues. Default value: ``false``.Project members with *View and assign to their company* and *Full visibility* can view deleted published and unpublished issues they originally created. Project members with Manage issues or Manage member permissions access can view all published issues that were deleted in a project. In addition, they can see unpublished deleted issues if they are an issue watcher, assignee, or creator.For more information about deleted issues see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Issues_Search_Filter>`_.
        filterdeleted: Optional[bool] = None

        # Filter issues by the chronological user-friendly identifier. Separate multiple values with commas.
        filterdisplay_id: Optional[int] = None

        # Filter issues by due date, in one of the following URL-encoded format: YYYY-MM-DD. Separate multiple values with commas. We support the following filtering options:- Date range: e.g., ``2022-03-02..2022-03-03``- Specific day: e.g., ``2022-03-02``- Specific start date: e.g.,  ``2022-03-02..``- Specific end date: e.g., ``..2022-03-02``For more details, see `JSON API Filtering <http://jsonapi.org/format/#fetching-filtering>`_.
        filterdue_date: Optional[str] = None

        # Filter issues by the unique issue ID. Separate multiple values with commas.
        filterid: Optional[list[str]] = None

        # Filter issues by the unique identifier of the type of the issue. Note that the API name for type is ``subtype``. Separate multiple values with commas.
        filterissue_subtype_id: Optional[list[str]] = None

        # Filter issues by the unique identifier of the category of the issue. Note that the API name for category is ``type``.  Separate multiple values with commas.
        filterissue_type_id: Optional[list[str]] = None

        # Retrieves pushpin issues associated with the specified files. We support all file types that are compatible with the Files tool. You need to specify the URL-encoded file item IDs. To find the file item IDs, use the `Data Management <en/docs/data/v2/developers_guide/overview/>`_ API.Note that you need to specify the 3D model item ID, which retrieves all pushpins associated with all 2D sheets and views associated with the 3D model. Similarly, if you specify a specific PDF file it retrieves all the pushpin issues associated with all the PDF file pages. We do not currently support retrieving pushpin issues associated with a specific 2D sheet or view.By default, it returns pushpins for the latest version of the file. To retrieve pushpins for a specific version of a file together with pushpins for all previous versions of the specified file version, specify the version number, in the following format: ``@[version-number]``.For example, ``filter[linkedDocument]=urn%3Aadsk.wipprod%3Adm.lineage%3AtFbo9zuDTW-nPh45gnM4gA@2``.Separate multiple values with commas.Note that we do not currently support filtering sheets from the ACC Build Sheets tool.
        filterlinked_document_urn: Optional[list[str]] = None

        # Retrieves issues associated with the specified location but not the location's sublocations. To also retrieve issues that relate to the locations's sublocations use the ``sublocationId`` filter. Separate multiple values with commas.
        filterlocation_id: Optional[list[str]] = None

        # Filter issues by the unique identifier of the type of root cause for the issue. Separate multiple values with commas.
        filterroot_cause_id: Optional[list[str]] = None

        # Filter issues using 'search' criteria. this will filter both title and issues display ID. For example, use filter[search]=300
        filtersearch: Optional[str] = None

        # Filter issues by start date, in one of the following URL-encoded format: YYYY-MM-DD. Separate multiple values with commas. We support the following filtering options:- Date range: e.g., ``2022-03-02..2022-03-03``- Specific day: e.g., ``2022-03-02``- Specific start date: e.g.,  ``2022-03-02..``- Specific end date: e.g., ``..2022-03-02``For more details, see `JSON API Filtering <http://jsonapi.org/format/#fetching-filtering>`_.
        filterstart_date: Optional[str] = None

        # Filter issues by their status. Separate multiple values with commas.
        filterstatus: Optional[str] = None

        # Retrieves issues associated with the specified unique LBS (Location Breakdown Structure) identifier, as well as issues associated with the sub locations of the LBS identifier. Separate multiple values with commas.
        filtersub_location_id: Optional[list[str]] = None

        # Filter issues updated at the specified date and time, in one of the following URL-encoded formats: YYYY-MM-DDThh:mm:ss.sz or YYYY-MM-DD. Separate multiple values with commas. We support the following filtering options:- Date range: e.g., ``2022-03-02..2022-03-03`` or ``2022-02-28T22:00:00.000Z..2022-03-28T22:00:00.000Z``- Specific day: e.g., ``2022-03-02`` or ``2022-02-28T22:00:00.000Z``- Specific start date: e.g.,  ``2022-03-02..`` or ``2022-02-28T22:00:00.000Z..``- Specific end date: e.g., ``..2022-03-02`` or ``..2022-02-28T22:00:00.000Z``For more details, see `JSON API Filtering <http://jsonapi.org/format/#fetching-filtering>`_.
        filterupdated_at: Optional[datetime.datetime] = None

        # Filter issues by the unique identifier of the user who updated the issue. Separate multiple values with commas. For Example: ``A3RGM375QTZ7``.
        filterupdated_by: Optional[list[str]] = None

        # Only return valid issues (=no empty type/subtype). Default value: ``undefined`` (meaning will return both valid & invalid issues).
        filtervalid: Optional[bool] = None

        # Return specified number of issues. Acceptable values are 1-100. Default value: ``100``.
        limit: Optional[int] = None

        # Return issues starting from the specified offset number. Default value: ``0``.
        offset: Optional[int] = None

        # Sort issues by specified fields. Separate multiple values with commas. To sort in descending order add a ``-`` (minus sign) before the sort criteria. Possible values: ``displayId``, ``title``, ``description``, ``status``, ``assignedTo``, ``assignedToType``, ``dueDate``, ``locationDetails``, ``published``, ``closedBy``, ``closedAt``, ``createdBy``, ``createdAt``, ``updatedAt``, ``issueSubType``, ``issueType``, ``customAttributes``, ``startDate``, ``rootCause``. For example: ``sortBy=status,-displayId,-dueDate,customAttributes[5c07cbe2-256a-48f1-b35b-2e5e00914104]``.
        sort_by: Optional[list[str]] = None

    
    @dataclass
    class IssuesRequestBuilderGetRequestConfiguration(RequestConfiguration[IssuesRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class IssuesRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from enum import Enum

class SearchRfisPostRequestBody_sort_field(str, Enum):
    CreatedAt = "createdAt",
    Status = "status",
    DueDate = "dueDate",
    Title = "title",
    Location = "location",
    UpdatedAt = "updatedAt",
    ClosedAt = "closedAt",
    CostImpact = "costImpact",
    ScheduleImpact = "scheduleImpact",
    Priority = "priority",
    Discipline = "discipline",
    Category = "category",
    Reference = "reference",
    CustomIdentifier = "customIdentifier",
    CustomAttribute = "customAttribute",




from enum import Enum

class ClassificationsImportPostRequestBody_classifications_measurementType(str, Enum):
    AREA = "AREA",
    COUNT = "COUNT",
    DISTANCE = "DISTANCE",
    VOLUME = "VOLUME",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .linked_files_get_response import LinkedFilesGetResponse

class LinkedFilesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/rcm/v1/projects/{projectId}/published-versions/{versionId}/linked-files
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new LinkedFilesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/rcm/v1/projects/{projectId}/published-versions/{versionId}/linked-files{?filter*,includeHost*,limit*,offset*}", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[LinkedFilesRequestBuilderGetQueryParameters]] = None) -> Optional[LinkedFilesGetResponse]:
        """
        Retrieves metadata and signed download URLs for a published version of a Revit (RVT) cloud model, whether workshared or non-workshared, and any Revit files linked to it.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[LinkedFilesGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .linked_files_get_response import LinkedFilesGetResponse

        return await self.request_adapter.send_async(request_info, LinkedFilesGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[LinkedFilesRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves metadata and signed download URLs for a published version of a Revit (RVT) cloud model, whether workshared or non-workshared, and any Revit files linked to it.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> LinkedFilesRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: LinkedFilesRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return LinkedFilesRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class LinkedFilesRequestBuilderGetQueryParameters():
        """
        Retrieves metadata and signed download URLs for a published version of a Revit (RVT) cloud model, whether workshared or non-workshared, and any Revit files linked to it.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "include_host":
                return "includeHost"
            if original_name == "filter":
                return "filter"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            return original_name
        
        # Specifies criteria for filtering the linked files by name or publish status.- Filter by file name:- Example: ``filter[name]=StructuralModel_2023.rvt&filter[name]=ArchitecturalModel_2023.rvt``- Filter by publish status:- Published files: ``filter[publishStatus]=published``- Unpublished files: ``filter[publishStatus]=notPublished``- Both published and unpublished: ``filter[publishStatus]=published,notPublished``
        filter: Optional[str] = None

        # Indicates whether to include the signed URL for the host model in the response.``true`` (default): the host model’s signed URL is included.``false``: only includes the signed URLs of the linked files.
        include_host: Optional[bool] = None

        # The maximum number of linked models to return in a single request. Maxium: ``600``. Default: ``600``.
        limit: Optional[int] = None

        # The index at which the endpoint starts returning results. Used for pagination. Default: ``0``.This is a zero-based index (if set to ``100``, results start from the 101st entry).
        offset: Optional[int] = None

    
    @dataclass
    class LinkedFilesRequestBuilderGetRequestConfiguration(RequestConfiguration[LinkedFilesRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .sub_cost_items_copy_post_request_body_from import SubCostItemsCopyPostRequestBody_from
    from .sub_cost_items_copy_post_request_body_source import SubCostItemsCopyPostRequestBody_source
    from .sub_cost_items_copy_post_request_body_to import SubCostItemsCopyPostRequestBody_to

@dataclass
class SubCostItemsCopyPostRequestBody(Parsable):
    # The type of the sub cost items being copied. Note that ``contract`` and ``budget`` can only be used as a source type, not a destination. Possible values: ``contract``, ``budget``, ``estimated``, ``proposed``, ``submitted``, ``approved``, ``committed``.
    from_: Optional[SubCostItemsCopyPostRequestBody_from] = None
    # The source object defining where sub-cost items are copied from. When copying from a ``budget`` or ``contract``, this must include one or more source IDs.
    source: Optional[list[SubCostItemsCopyPostRequestBody_source]] = None
    # The type to which the sub cost items are copied. Note that ``contract`` and ``budget`` cannot be used as a destination. Possible values: ``estimated``, ``proposed``, ``submitted``, ``approved``, ``committed``.
    to: Optional[SubCostItemsCopyPostRequestBody_to] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> SubCostItemsCopyPostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: SubCostItemsCopyPostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return SubCostItemsCopyPostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .sub_cost_items_copy_post_request_body_from import SubCostItemsCopyPostRequestBody_from
        from .sub_cost_items_copy_post_request_body_source import SubCostItemsCopyPostRequestBody_source
        from .sub_cost_items_copy_post_request_body_to import SubCostItemsCopyPostRequestBody_to

        from .sub_cost_items_copy_post_request_body_from import SubCostItemsCopyPostRequestBody_from
        from .sub_cost_items_copy_post_request_body_source import SubCostItemsCopyPostRequestBody_source
        from .sub_cost_items_copy_post_request_body_to import SubCostItemsCopyPostRequestBody_to

        fields: dict[str, Callable[[Any], None]] = {
            "from": lambda n : setattr(self, 'from_', n.get_enum_value(SubCostItemsCopyPostRequestBody_from)),
            "source": lambda n : setattr(self, 'source', n.get_collection_of_object_values(SubCostItemsCopyPostRequestBody_source)),
            "to": lambda n : setattr(self, 'to', n.get_enum_value(SubCostItemsCopyPostRequestBody_to)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_enum_value("from", self.from_)
        writer.write_collection_of_object_values("source", self.source)
        writer.write_enum_value("to", self.to)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .templates_get_response_results_steps_tasks_assigned_to_type import TemplatesGetResponse_results_steps_tasks_assignedToType

@dataclass
class TemplatesGetResponse_results_steps_tasks(Parsable):
    # The Autodesk ID or member group ID of the ``user``, ``company``, or ``role`` assigned to the task.
    assigned_to: Optional[str] = None
    # Specifies whether the task is assigned to a user, company, or role.Possible values: ``1`` (user), ``2`` (company), ``3`` (role).
    assigned_to_type: Optional[TemplatesGetResponse_results_steps_tasks_assignedToType] = None
    # The internal, globally unique identifier (UUID) for the task.
    id: Optional[UUID] = None
    # - ``true``: the task is required to complete the step.- ``false``: (default) the task is not required to complete the step.
    is_required: Optional[bool] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> TemplatesGetResponse_results_steps_tasks:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: TemplatesGetResponse_results_steps_tasks
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return TemplatesGetResponse_results_steps_tasks()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .templates_get_response_results_steps_tasks_assigned_to_type import TemplatesGetResponse_results_steps_tasks_assignedToType

        from .templates_get_response_results_steps_tasks_assigned_to_type import TemplatesGetResponse_results_steps_tasks_assignedToType

        fields: dict[str, Callable[[Any], None]] = {
            "assignedTo": lambda n : setattr(self, 'assigned_to', n.get_str_value()),
            "assignedToType": lambda n : setattr(self, 'assigned_to_type', n.get_enum_value(TemplatesGetResponse_results_steps_tasks_assignedToType)),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "isRequired": lambda n : setattr(self, 'is_required', n.get_bool_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("assignedTo", self.assigned_to)
        writer.write_enum_value("assignedToType", self.assigned_to_type)
        writer.write_uuid_value("id", self.id)
        writer.write_bool_value("isRequired", self.is_required)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .smartregister_get_response_spec_category import SmartregisterGetResponse_specCategory
    from .smartregister_get_response_target_group import SmartregisterGetResponse_targetGroup

@dataclass
class SmartregisterGetResponse(Parsable):
    # The division code associated with the submittal. This is the equivalent to the *Division* column in the UI.
    division_code: Optional[str] = None
    # The division name associated with the submittal. This is equivalent to the name in the *Division* filter in the UI.
    division_name: Optional[str] = None
    # The submittal sub section code. This is equivalent to the *Sub section number* column in the UI.
    para_code: Optional[str] = None
    # The type of specification category associated with the submittal. This is equivalent to *Submittal type* in the UI.Possible values: ``Test Reports``, ``Shop Drawings``, ``Schedules``, ``Samples``, ``Sample Warranty``, ``Reports``, ``Qualification Data``, ``QUALITY ASSURANCE``, ``Product Data``, ``Performance Data``, ``Mfg. Instructions``, ``Meeting/Conferences``, ``Drawings``, ``Delegated-Design``, ``Certifications``, ``Certificates``, ``Calculations``, ``Attic Stock``, ``Demonstrations``, ``General Warranties``, ``O&M Manuals``, ``Special Warranties``, ``LEED``, ``As-Builts``, ``TESTS AND INSPECTIONS``, ``General``, ``Manufacturers Instructions``, ``Substitutions``, ``Mix Design``, ``Others``
    spec_category: Optional[SmartregisterGetResponse_specCategory] = None
    # The CSI specification name of the submittal. This is equivalent of the *Section name* column in the UI.
    spec_name: Optional[str] = None
    # The CSI specification code of the submittal. This is equivalent to the *Section number* column in the UI.
    spec_number: Optional[str] = None
    # The description of the submittal. This is equivalent to the *Submittal description* column in the UI.
    submittal_description: Optional[str] = None
    # The index of the submittal in the submittal log. This is the equivalent to the *Submittal number* column in the UI.
    submittal_id: Optional[str] = None
    # The name of the submittal sub section. This is equivalent to *Sub section name* column in the UI.
    submittals_heading: Optional[str] = None
    # The submittal target date, in ISO 8601 format. This is equivalent to the *Target date* column in the UI.
    target_date: Optional[str] = None
    # The submittal group associcated with the submittal. This is equivalent to the *Submittal group* column in the UI.Possible values: ``ACTION AND INFORMATIONAL``, ``Closeout Submittals``, ``DIVISION 01 REQUIREMENTS``, ``Field Quality Control``, ``Mockups``, ``QUALITY ASSURANCE``, ``TESTS AND INSPECTIONS``
    target_group: Optional[SmartregisterGetResponse_targetGroup] = None
    # The user notes associcated with the submittal. This is equivalent to the *User notes* column in the UI.
    user_notes: Optional[str] = None
    # The version name provided by the user when creating the version.
    version_name: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> SmartregisterGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: SmartregisterGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return SmartregisterGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .smartregister_get_response_spec_category import SmartregisterGetResponse_specCategory
        from .smartregister_get_response_target_group import SmartregisterGetResponse_targetGroup

        from .smartregister_get_response_spec_category import SmartregisterGetResponse_specCategory
        from .smartregister_get_response_target_group import SmartregisterGetResponse_targetGroup

        fields: dict[str, Callable[[Any], None]] = {
            "divisionCode": lambda n : setattr(self, 'division_code', n.get_str_value()),
            "divisionName": lambda n : setattr(self, 'division_name', n.get_str_value()),
            "paraCode": lambda n : setattr(self, 'para_code', n.get_str_value()),
            "specCategory": lambda n : setattr(self, 'spec_category', n.get_enum_value(SmartregisterGetResponse_specCategory)),
            "specName": lambda n : setattr(self, 'spec_name', n.get_str_value()),
            "specNumber": lambda n : setattr(self, 'spec_number', n.get_str_value()),
            "submittalDescription": lambda n : setattr(self, 'submittal_description', n.get_str_value()),
            "submittalId": lambda n : setattr(self, 'submittal_id', n.get_str_value()),
            "submittalsHeading": lambda n : setattr(self, 'submittals_heading', n.get_str_value()),
            "targetDate": lambda n : setattr(self, 'target_date', n.get_str_value()),
            "targetGroup": lambda n : setattr(self, 'target_group', n.get_enum_value(SmartregisterGetResponse_targetGroup)),
            "userNotes": lambda n : setattr(self, 'user_notes', n.get_str_value()),
            "versionName": lambda n : setattr(self, 'version_name', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("divisionCode", self.division_code)
        writer.write_str_value("divisionName", self.division_name)
        writer.write_str_value("paraCode", self.para_code)
        writer.write_enum_value("specCategory", self.spec_category)
        writer.write_str_value("specName", self.spec_name)
        writer.write_str_value("specNumber", self.spec_number)
        writer.write_str_value("submittalDescription", self.submittal_description)
        writer.write_str_value("submittalId", self.submittal_id)
        writer.write_str_value("submittalsHeading", self.submittals_heading)
        writer.write_str_value("targetDate", self.target_date)
        writer.write_enum_value("targetGroup", self.target_group)
        writer.write_str_value("userNotes", self.user_notes)
        writer.write_str_value("versionName", self.version_name)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .item.responses_item_request_builder import ResponsesItemRequestBuilder
    from .responses_get_response import ResponsesGetResponse

class ResponsesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/submittals/v2/projects/{projectId}/responses
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new ResponsesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/submittals/v2/projects/{projectId}/responses{?limit*,offset*}", path_parameters)
    
    def by_id(self,id: str) -> ResponsesItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.submittals.v2.projects.item.responses.item collection
        param id: The ID of the submittal item to retrieve revisions for. To obtain this ID, call `GET items </en/docs/acc/v1/reference/http/submittals-items-GET/>`_.
        Returns: ResponsesItemRequestBuilder
        """
        if id is None:
            raise TypeError("id cannot be null.")
        from .item.responses_item_request_builder import ResponsesItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["id"] = id
        return ResponsesItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[ResponsesRequestBuilderGetQueryParameters]] = None) -> Optional[ResponsesGetResponse]:
        """
        Retrieves all the responses for the specified project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[ResponsesGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .responses_get_response import ResponsesGetResponse

        return await self.request_adapter.send_async(request_info, ResponsesGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[ResponsesRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves all the responses for the specified project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> ResponsesRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: ResponsesRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return ResponsesRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class ResponsesRequestBuilderGetQueryParameters():
        """
        Retrieves all the responses for the specified project.
        """
        # The maximum number of results per page. Possible values: ``1``- ``50``. Default value: ``20``. For example, to limit the response to two results per page, use ``limit=2``.
        limit: Optional[int] = None

        # The number of results to skip before starting to return data. For example, to skip the first 20 results, include ``offset=20`` in the query string. For more details, see the `JSON API Paging Help documentation <https://jsonapi.org/format/#fetching-pagination>`_.
        offset: Optional[int] = None

    
    @dataclass
    class ResponsesRequestBuilderGetRequestConfiguration(RequestConfiguration[ResponsesRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .issues_get_response_results_custom_attributes import IssuesGetResponse_results_customAttributes
    from .issues_get_response_results_gps_coordinates import IssuesGetResponse_results_gpsCoordinates
    from .issues_get_response_results_linked_documents import IssuesGetResponse_results_linkedDocuments
    from .issues_get_response_results_links import IssuesGetResponse_results_links
    from .issues_get_response_results_official_response import IssuesGetResponse_results_officialResponse
    from .issues_get_response_results_status import IssuesGetResponse_results_status

@dataclass
class IssuesGetResponse_results(Parsable):
    # The unique Autodesk ID of the member, company, or role of the current assignee for this issue. Note that if you select an assignee ID, you also need to select a type (``assignedToType``).
    assigned_to: Optional[str] = None
    # The type of the current assignee of this issue. Possible values: ``user``, ``company``, ``role``, ``null``. Note that if you select a type, you also need to select the assignee ID (``assignedTo``).
    assigned_to_type: Optional[str] = None
    # Not relevant
    attachment_count: Optional[int] = None
    # The date and time the issue was closed, in ISO8601 format.
    closed_at: Optional[datetime.datetime] = None
    # The unique identifier of the user who closed the issue.
    closed_by: Optional[str] = None
    # The number of comments in this issue.
    comment_count: Optional[int] = None
    # Not relevant
    container_id: Optional[UUID] = None
    # The date and time the issue was created, in ISO8601 format.
    created_at: Optional[datetime.datetime] = None
    # The unique identifier of the user who created the issue.
    created_by: Optional[str] = None
    # A list of custom attributes of the specific issue.
    custom_attributes: Optional[list[IssuesGetResponse_results_customAttributes]] = None
    # Indicates whether the issue was deleted. Default value: ``false``.Deleted issues can only be accessed by specific users. Project members with *View and assign to their company* and *Full visibility* permissions can view deleted published and unpublished issues they originally created. Project members with *Manage issues* or Manage member permissions access can view all published issues that were deleted in a project. In addition, they can see unpublished deleted issues if they are an issue watcher, assignee, or creator.For more information about deleted issues see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Issues_Search_Filter>`_.
    deleted: Optional[bool] = None
    # The date and time the issue was deleted, in ISO8601 format. This is only relevant for deleted issues.
    deleted_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who deleted the issue. This is only relevant for deleted issues.
    deleted_by: Optional[str] = None
    # A brief description of the issue and its purpose. Maximum 1000 characters.
    description: Optional[str] = None
    # The chronological user-friendly identifier of the issue.
    display_id: Optional[int] = None
    # The due date of the issue, in ISO8601 format.
    due_date: Optional[str] = None
    # A GPS Coordinate which represents the geo location of the issue.
    gps_coordinates: Optional[IssuesGetResponse_results_gpsCoordinates] = None
    # The unique identifier of the issue.
    id: Optional[UUID] = None
    # The unique identifier of the subtype of the issue.
    issue_subtype_id: Optional[UUID] = None
    # Not relevant
    issue_template_id: Optional[UUID] = None
    # The unique identifier of the type of the issue.
    issue_type_id: Optional[UUID] = None
    # Information about the files associated with issues (pushpins).
    linked_documents: Optional[list[IssuesGetResponse_results_linkedDocuments]] = None
    # Not relevant
    links: Optional[list[IssuesGetResponse_results_links]] = None
    # The location related to the issue, provided as plain text. Maximum 250 characters.
    location_details: Optional[str] = None
    # The unique LBS (Location Breakdown Structure) identifier that relates to the issue.
    location_id: Optional[UUID] = None
    # Not relevant
    official_response: Optional[IssuesGetResponse_results_officialResponse] = None
    # Not relevant
    opened_at: Optional[datetime.datetime] = None
    # Not relevant
    opened_by: Optional[str] = None
    # Not relevant
    owner_id: Optional[str] = None
    # The list of actions permitted for the user for this issue in its current state.Note that if a user with *View and assign to their company* permissions attempts to assign a user from a another company to the issue, it will return an error.Possible Values: ``assign_all`` (can assign another user from another company to the issue), ``assign_same_company`` (can only assign another user from the same company to the issue), ``clear_assignee``, ``delete``, ``add_comment``, ``add_attachment``, ``remove_attachment``.The following values are not relevant: ``add_attachment``, ``remove_attachment``.
    permitted_actions: Optional[list[str]] = None
    # A list of attributes the current user can manipulate in the current context. ``issueTypeId``, ``linkedDocument``, ``links``, ``ownerId``, ``officialResponse``, ``rootCauseId``, ``snapshotUrn`` are not applicable.Possible Values: ``title``, ``description``, ``issueTypeId``, ``issueSubtypeId``, ``status``, ``assignedTo``, ``assignedToType``, ``dueDate``, ``locationId``, ``locationDetails``, ``linkedDocuments``, ``links``, ``ownerId``, ``rootCauseId``, ``officialResponse``, ``customAttributes``, ``snapshotUrn``, ``startDate``, ``published``, ``deleted``, ``watchers``.
    permitted_attributes: Optional[list[str]] = None
    # A list of statuses accessible to the current user, this is based on the current status of the issue and the user permissions.Possible Values: ``open``, ``pending``, ``in_review``, ``closed``.
    permitted_statuses: Optional[list[str]] = None
    # States whether the issue is published. Default value: ``false`` (e.g. unpublished).
    published: Optional[bool] = None
    # The unique identifier of the type of root cause for the issue.
    root_cause_id: Optional[UUID] = None
    # Not relevant
    snapshot_has_markups: Optional[bool] = None
    # Not relevant
    snapshot_urn: Optional[str] = None
    # The start date of the issue, in ISO8601 format.
    start_date: Optional[str] = None
    # The current status of the issue.Possible values: ``draft``, ``open``, ``pending``, ``in_progress``, ``completed``, ``in_review``,  ``not_approved``, ``in_dispute``,  ``closed``.For more information about statuses, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Issues_Statuses>`_.
    status: Optional[IssuesGetResponse_results_status] = None
    # The title of the issue. Maximum 100 characters.
    title: Optional[str] = None
    # The date and time the issue was updated, in ISO8601 format.
    updated_at: Optional[datetime.datetime] = None
    # The unique identifier of the user who updated the issue.
    updated_by: Optional[str] = None
    # The list of watchers for the issue. To find the name of the watcher, call `GET users </en/docs/acc/v1/reference/http/users-GET>`_.
    watchers: Optional[list[str]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> IssuesGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: IssuesGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return IssuesGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .issues_get_response_results_custom_attributes import IssuesGetResponse_results_customAttributes
        from .issues_get_response_results_gps_coordinates import IssuesGetResponse_results_gpsCoordinates
        from .issues_get_response_results_linked_documents import IssuesGetResponse_results_linkedDocuments
        from .issues_get_response_results_links import IssuesGetResponse_results_links
        from .issues_get_response_results_official_response import IssuesGetResponse_results_officialResponse
        from .issues_get_response_results_status import IssuesGetResponse_results_status

        from .issues_get_response_results_custom_attributes import IssuesGetResponse_results_customAttributes
        from .issues_get_response_results_gps_coordinates import IssuesGetResponse_results_gpsCoordinates
        from .issues_get_response_results_linked_documents import IssuesGetResponse_results_linkedDocuments
        from .issues_get_response_results_links import IssuesGetResponse_results_links
        from .issues_get_response_results_official_response import IssuesGetResponse_results_officialResponse
        from .issues_get_response_results_status import IssuesGetResponse_results_status

        fields: dict[str, Callable[[Any], None]] = {
            "assignedTo": lambda n : setattr(self, 'assigned_to', n.get_str_value()),
            "assignedToType": lambda n : setattr(self, 'assigned_to_type', n.get_str_value()),
            "attachmentCount": lambda n : setattr(self, 'attachment_count', n.get_int_value()),
            "closedAt": lambda n : setattr(self, 'closed_at', n.get_datetime_value()),
            "closedBy": lambda n : setattr(self, 'closed_by', n.get_str_value()),
            "commentCount": lambda n : setattr(self, 'comment_count', n.get_int_value()),
            "containerId": lambda n : setattr(self, 'container_id', n.get_uuid_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "customAttributes": lambda n : setattr(self, 'custom_attributes', n.get_collection_of_object_values(IssuesGetResponse_results_customAttributes)),
            "deleted": lambda n : setattr(self, 'deleted', n.get_bool_value()),
            "deletedAt": lambda n : setattr(self, 'deleted_at', n.get_datetime_value()),
            "deletedBy": lambda n : setattr(self, 'deleted_by', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "displayId": lambda n : setattr(self, 'display_id', n.get_int_value()),
            "dueDate": lambda n : setattr(self, 'due_date', n.get_str_value()),
            "gpsCoordinates": lambda n : setattr(self, 'gps_coordinates', n.get_object_value(IssuesGetResponse_results_gpsCoordinates)),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "issueSubtypeId": lambda n : setattr(self, 'issue_subtype_id', n.get_uuid_value()),
            "issueTemplateId": lambda n : setattr(self, 'issue_template_id', n.get_uuid_value()),
            "issueTypeId": lambda n : setattr(self, 'issue_type_id', n.get_uuid_value()),
            "linkedDocuments": lambda n : setattr(self, 'linked_documents', n.get_collection_of_object_values(IssuesGetResponse_results_linkedDocuments)),
            "links": lambda n : setattr(self, 'links', n.get_collection_of_object_values(IssuesGetResponse_results_links)),
            "locationDetails": lambda n : setattr(self, 'location_details', n.get_str_value()),
            "locationId": lambda n : setattr(self, 'location_id', n.get_uuid_value()),
            "officialResponse": lambda n : setattr(self, 'official_response', n.get_object_value(IssuesGetResponse_results_officialResponse)),
            "openedAt": lambda n : setattr(self, 'opened_at', n.get_datetime_value()),
            "openedBy": lambda n : setattr(self, 'opened_by', n.get_str_value()),
            "ownerId": lambda n : setattr(self, 'owner_id', n.get_str_value()),
            "permittedActions": lambda n : setattr(self, 'permitted_actions', n.get_collection_of_primitive_values(str)),
            "permittedAttributes": lambda n : setattr(self, 'permitted_attributes', n.get_collection_of_primitive_values(str)),
            "permittedStatuses": lambda n : setattr(self, 'permitted_statuses', n.get_collection_of_primitive_values(str)),
            "published": lambda n : setattr(self, 'published', n.get_bool_value()),
            "rootCauseId": lambda n : setattr(self, 'root_cause_id', n.get_uuid_value()),
            "snapshotHasMarkups": lambda n : setattr(self, 'snapshot_has_markups', n.get_bool_value()),
            "snapshotUrn": lambda n : setattr(self, 'snapshot_urn', n.get_str_value()),
            "startDate": lambda n : setattr(self, 'start_date', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(IssuesGetResponse_results_status)),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "watchers": lambda n : setattr(self, 'watchers', n.get_collection_of_primitive_values(str)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("assignedTo", self.assigned_to)
        writer.write_str_value("assignedToType", self.assigned_to_type)
        writer.write_int_value("attachmentCount", self.attachment_count)
        writer.write_datetime_value("closedAt", self.closed_at)
        writer.write_str_value("closedBy", self.closed_by)
        writer.write_int_value("commentCount", self.comment_count)
        writer.write_uuid_value("containerId", self.container_id)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_collection_of_object_values("customAttributes", self.custom_attributes)
        writer.write_bool_value("deleted", self.deleted)
        writer.write_datetime_value("deletedAt", self.deleted_at)
        writer.write_str_value("deletedBy", self.deleted_by)
        writer.write_str_value("description", self.description)
        writer.write_int_value("displayId", self.display_id)
        writer.write_str_value("dueDate", self.due_date)
        writer.write_object_value("gpsCoordinates", self.gps_coordinates)
        writer.write_uuid_value("id", self.id)
        writer.write_uuid_value("issueSubtypeId", self.issue_subtype_id)
        writer.write_uuid_value("issueTemplateId", self.issue_template_id)
        writer.write_uuid_value("issueTypeId", self.issue_type_id)
        writer.write_collection_of_object_values("linkedDocuments", self.linked_documents)
        writer.write_collection_of_object_values("links", self.links)
        writer.write_str_value("locationDetails", self.location_details)
        writer.write_uuid_value("locationId", self.location_id)
        writer.write_object_value("officialResponse", self.official_response)
        writer.write_datetime_value("openedAt", self.opened_at)
        writer.write_str_value("openedBy", self.opened_by)
        writer.write_str_value("ownerId", self.owner_id)
        writer.write_collection_of_primitive_values("permittedActions", self.permitted_actions)
        writer.write_collection_of_primitive_values("permittedAttributes", self.permitted_attributes)
        writer.write_collection_of_primitive_values("permittedStatuses", self.permitted_statuses)
        writer.write_bool_value("published", self.published)
        writer.write_uuid_value("rootCauseId", self.root_cause_id)
        writer.write_bool_value("snapshotHasMarkups", self.snapshot_has_markups)
        writer.write_str_value("snapshotUrn", self.snapshot_urn)
        writer.write_str_value("startDate", self.start_date)
        writer.write_enum_value("status", self.status)
        writer.write_str_value("title", self.title)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_collection_of_primitive_values("watchers", self.watchers)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class WorkflowsPostResponse_additionalOptions(Parsable):
    """
    Workflow-level settings that control whether the initiator can modify certain fields when starting a review.
    """
    # (``Allow Initiators to edit the review`` in the UI). Indicates whether the initiator can adjust reviewer assignments and step durations.``true``: the initiator can change reviewer assignments and durations.``false``: (default) reviewers and durations are fixed.
    allow_initiator_to_edit: Optional[bool] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WorkflowsPostResponse_additionalOptions:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WorkflowsPostResponse_additionalOptions
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WorkflowsPostResponse_additionalOptions()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "allowInitiatorToEdit": lambda n : setattr(self, 'allow_initiator_to_edit', n.get_bool_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("allowInitiatorToEdit", self.allow_initiator_to_edit)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class FoldersGetResponse_results(Parsable):
    # The description of the folder.
    description: Optional[str] = None
    # Indicates whether the folder is deleted.``true`` – The folder is deleted, either directly or because its parent folder was deleted.``false`` – The folder is not deleted.
    is_deleted: Optional[bool] = None
    # The date and time when the folder was last modified, in ISO 8601 format.
    last_updated_at: Optional[datetime.datetime] = None
    # The name of the folder.
    name: Optional[str] = None
    # The Autodesk ID of the user who last modified the folder. For details about the user, call `GET user </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-userId-GET/>`_.
    updated_by: Optional[str] = None
    # The Autodesk ID of the user who last modified the folder.
    updated_by_name: Optional[str] = None
    # The URN of the folder.
    urn: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> FoldersGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: FoldersGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return FoldersGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "isDeleted": lambda n : setattr(self, 'is_deleted', n.get_bool_value()),
            "lastUpdatedAt": lambda n : setattr(self, 'last_updated_at', n.get_datetime_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "updatedByName": lambda n : setattr(self, 'updated_by_name', n.get_str_value()),
            "urn": lambda n : setattr(self, 'urn', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("description", self.description)
        writer.write_bool_value("isDeleted", self.is_deleted)
        writer.write_datetime_value("lastUpdatedAt", self.last_updated_at)
        writer.write_str_value("name", self.name)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_str_value("updatedByName", self.updated_by_name)
        writer.write_str_value("urn", self.urn)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .budgets_import_post_request_body_data import BudgetsImportPostRequestBody_data

@dataclass
class BudgetsImportPostRequestBody(Parsable):
    # ``true`` if you want to append the new budgets to the project.``false`` if you want to purge existing budgets before importing the new budgets.
    append: Optional[bool] = None
    # A list of budget.
    data: Optional[list[BudgetsImportPostRequestBody_data]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> BudgetsImportPostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: BudgetsImportPostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return BudgetsImportPostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .budgets_import_post_request_body_data import BudgetsImportPostRequestBody_data

        from .budgets_import_post_request_body_data import BudgetsImportPostRequestBody_data

        fields: dict[str, Callable[[Any], None]] = {
            "append": lambda n : setattr(self, 'append', n.get_bool_value()),
            "data": lambda n : setattr(self, 'data', n.get_collection_of_object_values(BudgetsImportPostRequestBody_data)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("append", self.append)
        writer.write_collection_of_object_values("data", self.data)
    




from enum import Enum

class WithAttachmentPatchResponse_categoryId(str, Enum):
    One = "1",
    Two = "2",
    Three = "3",
    Four = "4",
    Five = "5",
    Six = "6",
    Seven = "7",
    Eight = "8",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .projects_post_request_body_template_options import ProjectsPostRequestBody_template_options

@dataclass
class ProjectsPostRequestBody_template(Parsable):
    """
    Information about a project in the current user's account that is configured as a template from which to copy products and settings when creating a new project:- If you *include* this object in a `POST accounts/:accountId/projects </en/docs/acc/v1/reference/http/admin-accountsaccountidprojects-POST/>`_ request, the cloned project's products and settings will match those of the template project.- If you *omit* this object from a `POST accounts/:accountId/projects </en/docs/acc/v1/reference/http/admin-accountsaccountidprojects-POST/>`_ request, all of the current ACC account's products are added to the cloned project and activated.
    """
    # Information about what to include when cloning a project template.
    options: Optional[ProjectsPostRequestBody_template_options] = None
    # The ID of a project template in the current ACC account from which to clone the new project and copy products and settings.Note that you cannot create a project template from another project template, but you can base it on a production project. Set this field to the project ``id`` of the production project. The new project template is not completely configured for use, but it will get you started.For more information about project templates, see `Project Templates <https://help.autodesk.com/view/BUILD/ENU/?guid=Account_Admin_Project_Templates>`_ in the Build help.
    project_id: Optional[UUID] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ProjectsPostRequestBody_template:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ProjectsPostRequestBody_template
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ProjectsPostRequestBody_template()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .projects_post_request_body_template_options import ProjectsPostRequestBody_template_options

        from .projects_post_request_body_template_options import ProjectsPostRequestBody_template_options

        fields: dict[str, Callable[[Any], None]] = {
            "options": lambda n : setattr(self, 'options', n.get_object_value(ProjectsPostRequestBody_template_options)),
            "projectId": lambda n : setattr(self, 'project_id', n.get_uuid_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("options", self.options)
        writer.write_uuid_value("projectId", self.project_id)
    




from enum import Enum

class NamingStandardsGetResponse_definition_metadata_type(str, Enum):
    ALPHANUMERIC = "ALPHANUMERIC",
    NONNUMERIC_TEXT = "NONNUMERIC_TEXT",
    NUMERIC = "NUMERIC",
    ARRAY = "ARRAY",
    CLASSIFICATION = "CLASSIFICATION",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class ReviewsPostRequestBody_workflowOptions_copyFilesOptions(Parsable):
    """
    (``Copy approved files`` in the UI) Option for copying approved files when the review is complete.
    """
    # (``Then copy approved files to`` in the UI) The URN of the target folder where approved files will be copied.To find the folder URN, follow the first four steps of the `Upload Files to the ACC Files tool </en/docs/acc/v1/tutorials/files/upload-document-s3/>`_ tutorial and note the ``data.id`` field in the response. Use ``data.attributes.name`` or ``data.attributes.displayName`` to confirm you have the correct folder.
    folder_urn: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ReviewsPostRequestBody_workflowOptions_copyFilesOptions:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ReviewsPostRequestBody_workflowOptions_copyFilesOptions
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ReviewsPostRequestBody_workflowOptions_copyFilesOptions()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "folderUrn": lambda n : setattr(self, 'folder_urn', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("folderUrn", self.folder_urn)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .linked_files_get_response_host_file import LinkedFilesGetResponse_hostFile
    from .linked_files_get_response_linked_files import LinkedFilesGetResponse_linkedFiles

@dataclass
class LinkedFilesGetResponse(Parsable):
    # Metadata about the host model (the main Revit model that contains linked files). This object is only returned in the response when ``includeHost`` is set to ``true``.
    host_file: Optional[LinkedFilesGetResponse_hostFile] = None
    # A list of linked Revit (RVT) models associated with the requested published model version.This object contains a list of linked files and pagination details.
    linked_files: Optional[list[LinkedFilesGetResponse_linkedFiles]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> LinkedFilesGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: LinkedFilesGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return LinkedFilesGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .linked_files_get_response_host_file import LinkedFilesGetResponse_hostFile
        from .linked_files_get_response_linked_files import LinkedFilesGetResponse_linkedFiles

        from .linked_files_get_response_host_file import LinkedFilesGetResponse_hostFile
        from .linked_files_get_response_linked_files import LinkedFilesGetResponse_linkedFiles

        fields: dict[str, Callable[[Any], None]] = {
            "hostFile": lambda n : setattr(self, 'host_file', n.get_object_value(LinkedFilesGetResponse_hostFile)),
            "linkedFiles": lambda n : setattr(self, 'linked_files', n.get_collection_of_object_values(LinkedFilesGetResponse_linkedFiles)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("hostFile", self.host_file)
        writer.write_collection_of_object_values("linkedFiles", self.linked_files)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class SearchGetResponse(Parsable):
    # The about_me property
    about_me: Optional[str] = None
    # The account_id property
    account_id: Optional[UUID] = None
    # The address_line_1 property
    address_line_1: Optional[str] = None
    # The address_line_2 property
    address_line_2: Optional[str] = None
    # The city property
    city: Optional[str] = None
    # The company property
    company: Optional[str] = None
    # The company_id property
    company_id: Optional[UUID] = None
    # The company_name property
    company_name: Optional[str] = None
    # The country property
    country: Optional[str] = None
    # The created_at property
    created_at: Optional[datetime.datetime] = None
    # The default_role property
    default_role: Optional[str] = None
    # The default_role_id property
    default_role_id: Optional[str] = None
    # The email property
    email: Optional[str] = None
    # The first_name property
    first_name: Optional[str] = None
    # The id property
    id: Optional[UUID] = None
    # The image_url property
    image_url: Optional[str] = None
    # The industry property
    industry: Optional[str] = None
    # The job_title property
    job_title: Optional[str] = None
    # The last_name property
    last_name: Optional[str] = None
    # The last_sign_in property
    last_sign_in: Optional[datetime.datetime] = None
    # The name property
    name: Optional[str] = None
    # The nickname property
    nickname: Optional[str] = None
    # The phone property
    phone: Optional[str] = None
    # The postal_code property
    postal_code: Optional[str] = None
    # The role property
    role: Optional[str] = None
    # The state_or_province property
    state_or_province: Optional[str] = None
    # The status property
    status: Optional[str] = None
    # The uid property
    uid: Optional[str] = None
    # The updated_at property
    updated_at: Optional[datetime.datetime] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> SearchGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: SearchGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return SearchGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "about_me": lambda n : setattr(self, 'about_me', n.get_str_value()),
            "account_id": lambda n : setattr(self, 'account_id', n.get_uuid_value()),
            "address_line_1": lambda n : setattr(self, 'address_line_1', n.get_str_value()),
            "address_line_2": lambda n : setattr(self, 'address_line_2', n.get_str_value()),
            "city": lambda n : setattr(self, 'city', n.get_str_value()),
            "company": lambda n : setattr(self, 'company', n.get_str_value()),
            "company_id": lambda n : setattr(self, 'company_id', n.get_uuid_value()),
            "company_name": lambda n : setattr(self, 'company_name', n.get_str_value()),
            "country": lambda n : setattr(self, 'country', n.get_str_value()),
            "created_at": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "default_role": lambda n : setattr(self, 'default_role', n.get_str_value()),
            "default_role_id": lambda n : setattr(self, 'default_role_id', n.get_str_value()),
            "email": lambda n : setattr(self, 'email', n.get_str_value()),
            "first_name": lambda n : setattr(self, 'first_name', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "image_url": lambda n : setattr(self, 'image_url', n.get_str_value()),
            "industry": lambda n : setattr(self, 'industry', n.get_str_value()),
            "job_title": lambda n : setattr(self, 'job_title', n.get_str_value()),
            "last_name": lambda n : setattr(self, 'last_name', n.get_str_value()),
            "last_sign_in": lambda n : setattr(self, 'last_sign_in', n.get_datetime_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "nickname": lambda n : setattr(self, 'nickname', n.get_str_value()),
            "phone": lambda n : setattr(self, 'phone', n.get_str_value()),
            "postal_code": lambda n : setattr(self, 'postal_code', n.get_str_value()),
            "role": lambda n : setattr(self, 'role', n.get_str_value()),
            "state_or_province": lambda n : setattr(self, 'state_or_province', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_str_value()),
            "uid": lambda n : setattr(self, 'uid', n.get_str_value()),
            "updated_at": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("about_me", self.about_me)
        writer.write_uuid_value("account_id", self.account_id)
        writer.write_str_value("address_line_1", self.address_line_1)
        writer.write_str_value("address_line_2", self.address_line_2)
        writer.write_str_value("city", self.city)
        writer.write_str_value("company", self.company)
        writer.write_uuid_value("company_id", self.company_id)
        writer.write_str_value("company_name", self.company_name)
        writer.write_str_value("country", self.country)
        writer.write_datetime_value("created_at", self.created_at)
        writer.write_str_value("default_role", self.default_role)
        writer.write_str_value("default_role_id", self.default_role_id)
        writer.write_str_value("email", self.email)
        writer.write_str_value("first_name", self.first_name)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("image_url", self.image_url)
        writer.write_str_value("industry", self.industry)
        writer.write_str_value("job_title", self.job_title)
        writer.write_str_value("last_name", self.last_name)
        writer.write_datetime_value("last_sign_in", self.last_sign_in)
        writer.write_str_value("name", self.name)
        writer.write_str_value("nickname", self.nickname)
        writer.write_str_value("phone", self.phone)
        writer.write_str_value("postal_code", self.postal_code)
        writer.write_str_value("role", self.role)
        writer.write_str_value("state_or_province", self.state_or_province)
        writer.write_str_value("status", self.status)
        writer.write_str_value("uid", self.uid)
        writer.write_datetime_value("updated_at", self.updated_at)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .steps_get_response_pagination import StepsGetResponse_pagination
    from .steps_get_response_results import StepsGetResponse_results

@dataclass
class StepsGetResponse(Parsable):
    # Describes pagination details for the response, including information about the current page and navigation to other pages.
    pagination: Optional[StepsGetResponse_pagination] = None
    # Review steps associated with the specified submittal item.
    results: Optional[list[StepsGetResponse_results]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> StepsGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: StepsGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return StepsGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .steps_get_response_pagination import StepsGetResponse_pagination
        from .steps_get_response_results import StepsGetResponse_results

        from .steps_get_response_pagination import StepsGetResponse_pagination
        from .steps_get_response_results import StepsGetResponse_results

        fields: dict[str, Callable[[Any], None]] = {
            "pagination": lambda n : setattr(self, 'pagination', n.get_object_value(StepsGetResponse_pagination)),
            "results": lambda n : setattr(self, 'results', n.get_collection_of_object_values(StepsGetResponse_results)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("pagination", self.pagination)
        writer.write_collection_of_object_values("results", self.results)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .attachments.attachments_request_builder import AttachmentsRequestBuilder
    from .issues.issues_request_builder import IssuesRequestBuilder
    from .issue_attribute_definitions.issue_attribute_definitions_request_builder import IssueAttributeDefinitionsRequestBuilder
    from .issue_attribute_mappings.issue_attribute_mappings_request_builder import IssueAttributeMappingsRequestBuilder
    from .issue_root_cause_categories.issue_root_cause_categories_request_builder import IssueRootCauseCategoriesRequestBuilder
    from .issue_types.issue_types_request_builder import IssueTypesRequestBuilder
    from .users.users_request_builder import UsersRequestBuilder

class WithProjectItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/issues/v1/projects/{projectId}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new WithProjectItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/issues/v1/projects/{projectId}", path_parameters)
    
    @property
    def attachments(self) -> AttachmentsRequestBuilder:
        """
        The attachments property
        """
        from .attachments.attachments_request_builder import AttachmentsRequestBuilder

        return AttachmentsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def issue_attribute_definitions(self) -> IssueAttributeDefinitionsRequestBuilder:
        """
        The issueAttributeDefinitions property
        """
        from .issue_attribute_definitions.issue_attribute_definitions_request_builder import IssueAttributeDefinitionsRequestBuilder

        return IssueAttributeDefinitionsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def issue_attribute_mappings(self) -> IssueAttributeMappingsRequestBuilder:
        """
        The issueAttributeMappings property
        """
        from .issue_attribute_mappings.issue_attribute_mappings_request_builder import IssueAttributeMappingsRequestBuilder

        return IssueAttributeMappingsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def issue_root_cause_categories(self) -> IssueRootCauseCategoriesRequestBuilder:
        """
        The issueRootCauseCategories property
        """
        from .issue_root_cause_categories.issue_root_cause_categories_request_builder import IssueRootCauseCategoriesRequestBuilder

        return IssueRootCauseCategoriesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def issue_types(self) -> IssueTypesRequestBuilder:
        """
        The issueTypes property
        """
        from .issue_types.issue_types_request_builder import IssueTypesRequestBuilder

        return IssueTypesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def issues(self) -> IssuesRequestBuilder:
        """
        The issues property
        """
        from .issues.issues_request_builder import IssuesRequestBuilder

        return IssuesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def users(self) -> UsersRequestBuilder:
        """
        The users property
        """
        from .users.users_request_builder import UsersRequestBuilder

        return UsersRequestBuilder(self.request_adapter, self.path_parameters)
    

